class SSHKeydataObject:
    active_from = 'ActiveFrom'
    algorithm = 'Algorithm'
    allowed_source_restriction = 'AllowedSourceRestriction'
    approver = 'Approver'
    denied_source_restriction = 'DeniedSourceRestriction'
    device_guid = 'DeviceGuid'
    filepath = 'Filepath'
    forced_command = 'ForcedCommand'
    format = 'Format'
    is_encrypted = 'IsEncrypted'
    key_id = 'KeyId'
    keysetid = 'Keysetid'
    last_used = 'Last Used'
    length = 'Length'
    notes = 'Notes'
    options = 'Options '
    process_error = 'ProcessError'
    process_status = 'ProcessStatus'
    reason = 'Reason'
    type = 'Type'
    username = 'Username'
    violation_status = 'ViolationStatus'

