class PermissionParameter:
    is_associate_allowed = 'IsAssociateAllowed'
    is_create_allowed = 'IsCreateAllowed'
    is_delete_allowed = 'IsDeleteAllowed'
    is_manage_permissions_allowed = 'IsManagePermissionsAllowed'
    is_policy_write_allowed = 'IsPolicyWriteAllowed'
    is_private_key_read_allowed = 'IsPrivateKeyReadAllowed'
    is_private_key_write_allowed = 'IsPrivateKeyWriteAllowed'
    is_read_allowed = 'IsReadAllowed'
    is_rename_allowed = 'IsRenameAllowed'
    is_revoke_allowed = 'IsRevokeAllowed'
    is_view_allowed = 'IsViewAllowed'
    is_write_allowed = 'IsWriteAllowed'


class ProviderTypes:
    local = 'local'
    ad = 'AD'
    ldap = 'LDAP'
