class ConfigAttributeDataTypes:
    data_types = {
         0: 'None',
         1: 'Base64',
         2: 'Boolean',
         3: 'CaseExactString',
         4: 'CaseIgnoreList',
         5: 'CaseIgnoreString, String',
         6: 'Counter',
         7: 'DistinguishedName',
         8: 'EMailAddress',
         9: 'FaxNumber',
        10: 'Hold',
        11: 'Integer',
        12: 'Interval',
        13: 'IPNetworkAddress',
        14: 'NetworkAddress',
        15: 'NumericString',
        16: 'OctetList',
        17: 'OctetString',
        18: 'Path',
        19: 'PostalAddress',
        20: 'PrintableString',
        21: 'SchemaClass',
        22: 'Stream',
        23: 'TelephoneNumber',
        24: 'Time',
    }